calcRegularizers <- function(var.num.list, reg.map, 
                             xdata.train, ydata.train, 
                             xdata.test, ydata.test,
                             foldid, test.wit.all, res = NULL,
                             penalty = NULL,
                             mc.cores = 1) {
  if (is.null(res)) {
    res <- list()
  }
  temp.results  <- list()
  for (ix in seq_along(var.num.list)) {
    var.num <- var.num.list[ix]
    flog.info('Calculating for %s variables', var.num)
    temp.results[[ix]] <- lapply(names(reg.map), function(my.name) {
      # get an already cached model (if exists)
      var.name <- res[[my.name]][[length(res[[my.name]])]]
      if (!is.null(var.name)) {
        results <- var.name$results
        flog.info('Using cache in %s', my.name)
      } else {
        results = NULL
        flog.info('Could not find %s, going to generate it', my.name)
        flog.info('Calculate regularization for %s with target %s variables.', my.name, var.num)
        flog.info('  alpha = %.3f | xdata.train size %dx%d | xdata.test %dx%d', 
                  reg.map[[my.name]], nrow(xdata.train), ncol(xdata.train), nrow(xdata.test), ncol(xdata.test))
        flog.info('  test with all: %s', test.wit.all)
      }
      # upper case
      graph.name <- toupper(my.name)
      # remove alpha suffix
      graph.name <- gsub('[.][0-9]+', '', graph.name)
      # add alpha = alpha value
      graph.name <- paste0(graph.name, ' with alpha = ', reg.map[[my.name]])
      my.res <- calculate.reg(reg.map[[my.name]], graph.name,
                              xdata.train, ydata.train, xdata.test, ydata.test, 
                              foldid, var.num = var.num, results = results,
                              test.with.all = test.with.all, my.penalty = penalty,
                              mc.cores = mc.cores)
      if (is.null(results)) {
        res[[my.name]]            <<- list()
        res[[my.name]][[var.num]] <<- my.res
      }
      return(my.res)
    })
  }
  
  #
  #
  # convert to res object
  for(ix.var in seq_along(var.num.list)) {
    for(ix.map in seq_along(reg.map)) {
      if (is.null(res[[names(reg.map)[ix.map]]])) {
        res[[names(reg.map)[ix.map]]] <- list()
      }
      res[[names(reg.map)[ix.map]]][[var.num.list[ix.var]]] <- temp.results[[ix.var]][[ix.map]]
    }
  }
  return(res)
}