runMultipleTimes <- function(ntimes, reg.names, xdata, ydata, event.ix, censo.ix,
                             var.num = 30,
                             training.perc = .90, nfolds = 10, test.with.all = F,
                             network.normalization = NULL, 
                             xdata.normalizationType = 'colCenterScale',
                             centrality.normalization.type = 'max', mc.cores = 1) {
  #
  # create template for each row of the output
  col.names <- c('ix', 'type', 'pvalue', 'error', rownames(xdata))
  coef.template <- matrix(0, ncol = length(col.names), nrow = 1)
  #
  coef.template <- as.data.frame(coef.template)
  colnames(coef.template) <- col.names
  #
  #
  # store initial log threshold
  old.flog <- flog.threshold()
  #
  #
  # set a initial seed and then create a vector to control each run
  my.seeds <- unique(round(runif(ntimes*2)*100000))[seq(ntimes)]
  #
  all.results <- mclapply(seq(ntimes), function(ix) {
    training.seed <- my.seeds[ix]
    flog.info('Running ntime = %d/%d', ix, ntimes)
    #
    #
    # only log warnings or errors for function below
    flog.threshold(WARN)
    # get data
    set.seed(training.seed + 1)
    result <- extract.balanced.xdata.ydata.network.foldid(xdata, ydata, event.ix, censo.ix,
                                                          train.perc = training.set, nfolds = nfolds,
                                                          test.with.all = test.with.all,
                                                          network.normalizationType = network.normalizationType,
                                                          xdata.normalizationType = xdata.normalizationType,
                                                          centrality.normalizationType = centrality.normalizationType,
                                                          training.seed = my.seeds[ix])
    # assign all variables from the function
    for (var.name in names(result))
      assign(var.name, result[[var.name]])
    rm(result)
    gc()
    gc()
    gc()
    # start logging messages again
    flog.threshold(old.flog)
    # convert from years to months
    ydata.test$time  <- ydata.test$time * 12
    ydata.train$time <- ydata.train$time * 12
    #
    out.mat <- NULL
    for(name in names(reg.names)) {
      flog.info('ix: %d | type: %s | xdata: %dx%d', ix, name, nrow(xdata.train), ncol(xdata.train))
      out.tmp            <- coef.template
      out.tmp[1,'type']  <- name
      out.tmp[1,'ix']    <- ix
      out.tmp[1,'error'] <- tryCatch({
        results <- calculate.reg(reg.names[[name]]$alpha, name,
                                         xdata.train, ydata.train,
                                         xdata.test, ydata.test,
                                         foldid, 
                                         reg.names[[name]]$my.penalty, 
                                         nlambda = nlambda, test.with.all = T,
                                         var.num = var.num)
        #
        out.tmp['pvalue']                  <- results$km$pvalue
        out.tmp[results$var.names] <- as.vector(results$cox.model$coefficients)
        #
        rm(results)
        gc()
        gc()
        FALSE
      }, error = function(e) {
        flog.warn('Warning (ix: %d): failed to get non-zero solution in %s -- message: %s', ix, name, e)
        TRUE
      }
      )
      if (is.null(out.mat)) {
        out.mat <- out.tmp
      } else {
        out.mat <- rbind(out.mat, out.tmp)
      }
      #
    }
    return(out.mat)
  }, mc.cores = mc.cores)
  #
  # Convert all.results to matrix
  all.results.matrix <- c()
  for (ix in seq(length(all.results))) {
    all.results.matrix <- rbind(all.results.matrix, all.results[[ix]])
  }
  return(all.results.matrix)
}