SI
================

-   [Data for SI project](#data-for-si-project)
    -   [Getting data](#getting-data)
    -   [Description of data](#description-of-data)
    -   [Duplicate cases](#duplicate-cases)
-   [Survival analysis](#survival-analysis)

Data for SI project
===================

[Download gene expression data](http://sels.tecnico.ulisboa.pt/burn/si_32.csv)

[Download clinical data](http://sels.tecnico.ulisboa.pt/burn/si_32_clinical.csv)

Getting data
------------

Genes were selected from initial `55682` genes by:

1.  Runnnig survival models - Cox Proportional Hazard models - 10 times with different training sets
    -   We used two types of sparse penalization, [LASSO](http://statweb.stanford.edu/~tibs/lasso.html) and [Elastic Net](http://onlinelibrary.wiley.com/doi/10.1111/j.1467-9868.2005.00503.x/abstract) with `alpha = 0.5`

2.  Identify the selected genes
3.  Sum the number of times genes were selected
4.  Keep only the 32 genes selected the most

*note:* This is done in a different script outside this document.

``` r
# generated in analysis.Rmd
load('data/analysis.RData')
#
# from brca.data rpackage
data('clinical')
#
# From an iteration in interesting-genes.Rmd 
genes <- c('ENSG00000064042', 'ENSG00000066084', 'ENSG00000096006', 'ENSG00000102144', 'ENSG00000102893', 
           'ENSG00000102978', 'ENSG00000114757', 'ENSG00000120057', 'ENSG00000120256', 'ENSG00000120265', 
           'ENSG00000121871', 'ENSG00000123496', 'ENSG00000130720', 'ENSG00000132952', 'ENSG00000138653', 
           'ENSG00000152684', 'ENSG00000164309', 'ENSG00000170835', 'ENSG00000171199', 'ENSG00000171209', 
           'ENSG00000188549', 'ENSG00000196503', 'ENSG00000196517', 'ENSG00000203785', 'ENSG00000211911', 
           'ENSG00000220804', 'ENSG00000229204', 'ENSG00000241755', 'ENSG00000253802', 'ENSG00000270547', 
           'ENSG00000273237', 'ENSG00000279207')

# Select only the 32 genes
si <- xdata[genes,]

# Select only patients in the dataset
si.clinical <- clinical$all[unique(getParticipantCode(colnames(si))),]

# Write files to disk
write.table(si, file = 'si_32.csv', sep = '\t', col.names = T, row.names = T)
write.table(si.clinical, file = 'si_32_clinical.csv', sep = '\t', col.names = T, row.names = T)
```

Description of data
-------------------

-   Rows: Gene identifier using *Ensembl Gene ID*
-   Column: Tissue sample taken from a patient *(using tcga code)*
    -   *Note that there are biological replicates (i.e. same patient has multiple samples of tissue)*
-   Cell: Gene expression level using FPKM

**TCGA code description**

![TCGA code description](https://wiki.nci.nih.gov/download/attachments/39294833/barcode.png?version=1&modificationDate=1300400318000&api=v2)

Duplicate cases
---------------

``` r
# Get only the patient ID (first 12 characters)
cases   <- getParticipantCode(colnames(si))
# Find duplicated
my.dups <- cases[duplicated(cases)]
# Show duplicated
flog.info('Patients with more than a single replicates of tissue sample', sort(colnames(si)[cases %in% my.dups]), capture = T)
```

    ## INFO [2017-01-02 18:37:42] Patients with more than a single replicates of tissue sample
    ## 
    ##  [1] "TCGA-A7-A0DB-01A-11R-A00Z-07" "TCGA-A7-A0DB-01A-11R-A277-07"
    ##  [3] "TCGA-A7-A0DB-01C-02R-A277-07" "TCGA-A7-A0DC-01A-11R-A00Z-07"
    ##  [5] "TCGA-A7-A0DC-01B-04R-A22O-07" "TCGA-A7-A13D-01A-13R-A12P-07"
    ##  [7] "TCGA-A7-A13D-01A-13R-A277-07" "TCGA-A7-A13D-01B-04R-A277-07"
    ##  [9] "TCGA-A7-A13E-01A-11R-A12P-07" "TCGA-A7-A13E-01A-11R-A277-07"
    ## [11] "TCGA-A7-A13E-01B-06R-A277-07" "TCGA-A7-A26E-01A-11R-A169-07"
    ## [13] "TCGA-A7-A26E-01A-11R-A277-07" "TCGA-A7-A26E-01B-06R-A277-07"
    ## [15] "TCGA-A7-A26J-01A-11R-A169-07" "TCGA-A7-A26J-01A-11R-A277-07"
    ## [17] "TCGA-A7-A26J-01B-02R-A277-07"

Survival analysis
=================

Run the survival model with 10 fold cross validation to select the parameters

``` r
source('functions/extract_data.R')
if (ncol(si) == nrow(ydata)) {
  si.xdata <- t(si)
} else {
  si.xdata <- si
}
# time to months
ydata$time <- ydata$time / 365 * 12 # days to months

# Generate a balanced dataset
my.foldid <- gen.foldid(which(ydata$status), which(!ydata$status), 10)
result    <- calculate.reg(0, 'SI (with ridge)', si.xdata, ydata, si.xdata, ydata, my.foldid, mc.cores = 1, nlambda = 200)
# Show the results
result$km$plot
```

![](si_files/figure-markdown_github/survival-1.png)
